<!-- Generated by cargo-onedoc v0.2.2. DO NOT EDIT. -->

# anycase

[![Crates.io Version](https://badgers.space/crates/version/anycase)](https://crates.io/crates/anycase)
[![Docs.rs Latest](https://badgers.space/badge/docs.rs/latest/blue)](https://docs.rs/anycase)
[![Build Status](https://badgers.space/github/checks/rossmacarthur/anycase?label=build)](https://github.com/rossmacarthur/anycase/actions/workflows/rust.yaml)

💼 A case conversion library for Rust.

## 🚀 Getting started

First, add the `anycase` crate to your Cargo manifest.

```sh
cargo add anycase
```

Then you can use the `as_<case>` function to get a `Display` type.

```rust
let s = format!("snake case: {}", anycase::as_snake("Hello world!"));
assert_eq!(s, "snake case: hello_world");
```

Alternatively, you can use the `to_<case>` function to get a `String`.

```rust
let s = anycase::to_snake("Hello world!");
assert_eq!(s, "hello_world");
```

## 🤸 Usage

The `anycase` crate provides a set of functions to convert strings between
different case styles. The following cases are available.

Given an input of `Hello world!`:

- [`as_camel`][as_camel]           displays `helloWorld`
- [`as_pascal`][as_pascal]          displays `HelloWorld`
- [`as_snake`][as_snake]           displays `hello_world`
- [`as_screaming_snake`][as_screaming_snake] displays `HELLO_WORLD`
- [`as_kebab`][as_kebab]           displays `hello-world`
- [`as_screaming_kebab`][as_screaming_kebab] displays `HELLO_WORLD`
- [`as_train`][as_train]           displays `Hello-World`
- [`as_lower`][as_lower]           displays `hello world`
- [`as_title`][as_title]           displays `Hello World`
- [`as_upper`][as_upper]           displays `HELLO WORLD`

For all of the above functions, you can use the `to_<case>` variant to get a
`String` instead of a `Display` type.

Additionally, the crate provides the `raw` module containing the raw
functions which can be used to implement custom case conversion functions.

```rust
use anycase::raw;

let input = "Hello world!";
let output =  raw::to_string(input, raw::write_upper, raw::delim_fn("."));
assert_eq!(output, "HELLO.WORLD");
```

See the [module level documentation][crate::raw] for more details.

## How does it work?

This implementation divides the input string into words and applies a “word
function” to each word and calls a “delimiter function” for each word
boundary (the space between words).

Word boundaries are defined as follows:

- A set of consecutive non-letter/number/symbol e.g. `foo _bar` is two words
  `foo` and `bar`.
- A transition from a lowercase letter to an uppercase letter e.g. `fooBar`
  is two words `foo` and `Bar`.
- The second last uppercase letter in a word with multiple uppercase letters
  e.g. `FOOBar` is two words `FOO` and `Bar`.

The following `char` methods are used in the above conditions:

- [`char::is_alphanumeric`][char::is_alphanumeric] is used to determine if a character is a
  letter/number/symbol
- [`char::is_lowercase`][char::is_lowercase] is used to determine if a character is a lowercase
  letter
- [`char::is_uppercase`][char::is_uppercase] is used to determine if a character is an uppercase
  letter

## Features

This crate is designed to be `no_std` compatible. This is made possible by
disabling all default features. The following features are available:

- **`std`** *(enabled by default)* — Currently only enables the `alloc`
  feature but is here to allow for forward compatibility with any
  [`std`]\-only features.

- **`alloc`** — Links the [`alloc`] crate and enables the use of `String`
  functions.

## MSRV

The minimum supported Rust version (MSRV) is 1.56.0.  The policy of this
crate is to only increase the MSRV in a breaking release.

[`std`]: http://doc.rust-lang.org/std/
[`alloc`]: http://doc.rust-lang.org/alloc/

## License

This project is distributed under the terms of both the MIT license and the Apache License (Version 2.0).

See [LICENSE-APACHE](LICENSE-APACHE) and [LICENSE-MIT](LICENSE-MIT) for details.


[as_camel]: https://docs.rs/anycase/latest/anycase/fn.as_camel.html
[as_kebab]: https://docs.rs/anycase/latest/anycase/fn.as_kebab.html
[as_lower]: https://docs.rs/anycase/latest/anycase/fn.as_lower.html
[as_pascal]: https://docs.rs/anycase/latest/anycase/fn.as_pascal.html
[as_screaming_kebab]: https://docs.rs/anycase/latest/anycase/fn.as_screaming_kebab.html
[as_screaming_snake]: https://docs.rs/anycase/latest/anycase/fn.as_screaming_snake.html
[as_snake]: https://docs.rs/anycase/latest/anycase/fn.as_snake.html
[as_title]: https://docs.rs/anycase/latest/anycase/fn.as_title.html
[as_train]: https://docs.rs/anycase/latest/anycase/fn.as_train.html
[as_upper]: https://docs.rs/anycase/latest/anycase/fn.as_upper.html
[char::is_alphanumeric]: https://doc.rust-lang.org/stable/core/primitive.char.html#method.is_alphanumeric
[char::is_lowercase]: https://doc.rust-lang.org/stable/core/primitive.char.html#method.is_lowercase
[char::is_uppercase]: https://doc.rust-lang.org/stable/core/primitive.char.html#method.is_uppercase
[crate::raw]: https://doc.rs/anycase/latest/anycase/raw/index.html
